module Tests where

import Numeric.Natural

import Test.Tasty
import Test.Tasty.HUnit

import Lagerverwaltung

run = defaultMain $ testGroup "All tests" [enthaeltTests, hinzufuegenTests, teileVorhandenTests]

type LS = Lager String

einfach :: LS
einfach = l where
    l = [(A "A" 2), (A "B" 4), (A "C" 1), (A "D" 2), (A "E" 1)]
    
p1 = P "Kann" [(A "A" 1), (A "C" 1), (A "D" 2)]

p2 = P "Nicht" [(A "A" 2), (A "C" 3), (A "E" 1)]

p3 = P "KannNachP1" [(A "B" 2), (A "A" 1)]

-- Zählt die Gesamtzahl der Items in einem Lager 
anzahlItems:: Lager a -> Natural 
anzahlItems [] = 0
anzahlItems ((A _ n): ls) = n + anzahlItems ls
    
enthaeltTests = testGroup "Tests für enthaelt"
    [ testCase "D enthalten" $
      enthaelt einfach "D" == 2 @? "Enthaltender Eintrag nicht gefunden"
    , testCase "F nicht enthalten" $
      enthaelt einfach "F" == 0 @? "Eintrag sollte nicht enthalten sein" 
    ]

hinzufuegenTests = testGroup "Tests für hinzufuegen" 
    [ testCase "F hinzufügen, nicht enthalten" $
      enthaelt (hinzufuegen einfach "F" 4) "F" == 4 @? "Item nicht hinzugefügt"
    , testCase "D hinzufügen, enthalten" $
      enthaelt (hinzufuegen einfach "D" 2) "D" == 4 @? "Item nicht hinzugefügt"
    ]
    
entfernenTests = testGroup "Tests für entfernen"
    [ testCase "D entfernen, noch enthalten" $
      enthaelt (entfernen einfach "D" 1) "D" == 1 @? "Item nicht entfernt"
    , testCase "D entfernen, nicht mehr enthalten" $
      enthaelt (entfernen einfach "D" 2) "D" == 0 @? "Item nicht entfernt"
    , testCase "F entfernen, nicht im Lager" $
      enthaelt (entfernen einfach "F" 1) "F" == 0 @? "Item nicht entfernt"
    , testCase "D entfernen, noch enthalten, Gesamtzahl" $
      anzahlItems (entfernen einfach "D" 1) == anzahlItems einfach - 1 @? "Die Gesamtzahl passt nicht"
    , testCase "D entfernen, nicht mehr enthalten, Gesamtzahl" $
      anzahlItems (entfernen einfach "D" 2) == anzahlItems einfach - 2 @? "Die Gesamtzahl passt nicht"
    , testCase "F entfernen, nicht im Lager, Gesamtzahl" $
      anzahlItems (entfernen einfach "F" 1) == anzahlItems einfach @? "Die Gesamtzahl passt nicht"
    ]

teileVorhandenTests = testGroup "Tests für teileVorhanden"
    [ testCase "Produkt teile enthalten" $
      teileVorhanden p1 einfach == True @? "Teile nicht enthalten"
    , testCase "Produkt teile nicht enthalten" $
      teileVorhanden p2 einfach == False @? "Teile enthalten"
    ]
    
produzierenTests = testGroup "Tests für Produzieren"
    [ testCase "alle können Produziert werden" $
      anzahlItems (produzieren [p1, p3] einfach) == 3 @? "Teile von Produkte wurde nicht entfernt"
    , testCase "keiner kann Produziert werden" $
      anzahlItems (produzieren [p2, p2] einfach) == anzahlItems einfach @? "Teile von Produkte wurden entfernt"
    ]
